@@ -1,10 +1,12 @@
+from datetime import datetime
 from time import sleep
 
+import pytz
 from pyqalx import Bot, Set, Group
 from pyqalx.bot import QalxJob
 
 from qalx_orcaflex.bots.batch.summaries import ResultsSummariser
-from qalx_orcaflex.data_models import BatchOptions
+from qalx_orcaflex.data_models import BatchOptions, rehydrate_dataclass
 
 
 def process_batch(job: QalxJob):
@@ -14,7 +16,7 @@ def process_batch(job: QalxJob):
     """
     job.entity["meta"]["state"] = "processing"
     job.save_entity()
-    options = BatchOptions(**job.entity["meta"]["options"])
+    options = rehydrate_dataclass(BatchOptions, job.entity["meta"]["options"])
     tasks = job.entity["sets"]
     if not job.entity["meta"].get(
         "sims_submitted"
@@ -29,6 +31,12 @@ def process_batch(job: QalxJob):
             s["meta"]["state"]["info"] = options.sim_queue
             job.session.set.save(s)
             job.log.debug(f"added {s['guid']} to {options.sim_queue}.")
+
+        if options.notifications and options.notifications.notify_submitted:
+            # Send submitted notification if specified
+            options.notifications.notify_submitted.send(
+                entity=job.entity, session=job.session
+            )
         job.entity["meta"][
             "sims_submitted"
         ] = True  # set the flag so next time around we don't resubmit them
@@ -52,16 +60,16 @@ def send_batch_to(job: QalxJob, options: BatchOptions):
 
 
 def post_process_batch(job: QalxJob):
+    # Reload the entity without unpacking to ensure it's on the correct state
     job.reload_entity()
-    options = BatchOptions(**job.entity["meta"]["options"])
-    tasks = job.entity["sets"]
-    if not isinstance(list(tasks.values())[0], Set):
-        tasks = {s: job.session.set.get(guid) for s, guid in tasks.items()}
-    # Here we check to see if all the jobs have been processes by SimBots (or children)
-    process_sets = [
-        job.session.set.reload(e)["meta"].get("processing_complete")
-        for s, e in tasks.items()
-    ]
+    # Reload again, but this time unpack everything.  We use the results from the unpacked
+    # sets to check to see if all the jobs have been processed by SimBots (or children)
+    reloaded_entity = job.session.group.reload(
+        job.entity, unpack=True, fields=["sets", "meta"]
+    )
+    options = rehydrate_dataclass(BatchOptions, job.entity["meta"]["options"])
+    tasks = reloaded_entity["sets"]
+    process_sets = [task["meta"].get("processing_complete") for task in tasks.values()]
     job.log.debug(process_sets)
     # if they are all complete and we are supposed to do a results summary then crack on
     if (
@@ -78,19 +86,43 @@ def post_process_batch(job: QalxJob):
         job.log.debug(f"batch {job.e['guid']} summary complete.")
         job.save_entity()
         send_batch_to(job, options)
+        if options.notifications and options.notifications.notify_completed:
+            # Send completed notification if specified
+            options.notifications.notify_completed.send(
+                entity=job.entity, session=job.session
+            )
     elif options.summarise_results and (not job.entity["meta"].get("results_summary")):
         # if we do want to summarise this batch then let's wait for a bit and
         # then submit this job back to the batch
         # queue. It will eventually get back to this point and check again if
         # all jobs are complete.
-        sleep(options.wait_between_completion_checks)
-        batch_queue = job.session.queue.get_or_create(options.batch_queue)
-        Group.add_to_queue(payload=job.entity, queue=batch_queue)
-        job.log.debug(f"put batch {job.e['guid']} back on {options.batch_queue}.")
+        utc_now = datetime.utcnow().replace(tzinfo=pytz.utc)
+        has_timed_out = (
+            options.timeout
+            and (utc_now - job.entity["info"]["created"]["on"]).seconds
+            > options.timeout
+        )
+        if has_timed_out:
+            # The batch has timed out.  Send a notification if specified but do not add
+            # the batch back onto the queue to continue processing
+            if options.notifications and options.notifications.notify_timed_out:
+                options.notifications.notify_timed_out.send(
+                    entity=job.entity, session=job.session
+                )
+        else:
+            sleep(options.wait_between_completion_checks)
+            batch_queue = job.session.queue.get_or_create(options.batch_queue)
+            Group.add_to_queue(payload=job.entity, queue=batch_queue)
+            job.log.debug(f"put batch {job.e['guid']} back on {options.batch_queue}.")
     else:  # we've done all we were supposed to do here
         job.entity["meta"]["state"] = "processing_complete"
         job.save_entity()
         send_batch_to(job, options)
+        if options.notifications and options.notifications.notify_completed:
+            # Send completed notification if specified
+            options.notifications.notify_completed.send(
+                entity=job.entity, session=job.session
+            )
 
 
 class BatchBot(Bot):
