@@ -106,6 +106,24 @@ def preprocess_orcaflex(job: QalxJob):
     )
 
 
+def update_job_progress(ofx, update_queue, job):
+    """update the progress and save the item"""
+    time_of_last_update = time.time()
+    if ofx.progress:
+        should_exit = False
+        while not should_exit:
+            progress = update_queue.get()
+            update_queue.task_done()
+            if progress is None:
+                should_exit = True
+            else:
+                time_since_last_update = time.time() - time_of_last_update
+                if time_since_last_update > job.options.update_interval:
+                    ofx.progress["data"] = progress
+                    job.session.item.save(ofx.progress)
+                    time_of_last_update = time.time()
+
+
 # DEV_NOTE: this function was written a long time ago using a functional approach, as it has grown it probably should
 # have been converted to a more object-oriented approach. One day maybe we will get there.
 def process_orcaflex(job: QalxJob) -> None:
@@ -120,20 +138,7 @@ def process_orcaflex(job: QalxJob) -> None:
     )  # unpack the raw Set into our pre-defined model
     update_queue = queue.Queue()
 
-    def update_job_progress() -> None:
-        """update the progress and save the item"""
-        time_of_last_update = time.time()
-        if ofx.progress:
-            while True:
-                progress = update_queue.get()
-                time_since_last_update = time.time() - time_of_last_update
-                update_queue.task_done()
-                if time_since_last_update > job.options.update_interval:
-                    ofx.progress["data"] = progress
-                    job.session.item.save(ofx.progress)
-                    time_of_last_update = time.time()
-
-    threading.Thread(target=update_job_progress, daemon=True).start()
+    threading.Thread(target=update_job_progress, args=(ofx, update_queue, job)).start()
 
     def update_job_warnings(warnings: Sequence[str]) -> None:
         """add all the warning texts, save the item"""
@@ -524,7 +529,9 @@ def process_orcaflex(job: QalxJob) -> None:
     # this allows `postprocessing_function` to access model data
     job.context["model"] = model
     job.entity["meta"]["processing_complete"] = True  # we done.
-    # update_queue.join() # wait until all the progress updates are pushed
+    # Signal the end of update worker thread and release the memory
+    update_queue.put(None)
+    update_queue.join()
 
 
 def post_process_orcaflex(job: QalxJob):
@@ -534,3 +541,10 @@ def post_process_orcaflex(job: QalxJob):
             queue = job.session.queue.get_or_create(queue_name)
             job.entity.__class__.add_to_queue(payload=job.entity, queue=queue)
             job.log.debug(f"{job.entity.guid} sent to {queue_name}")
+    # Clear up the model and delete references to the model and context dict
+    # from the job. This is to try and keep the memory usage of the worker as
+    # low as possible for the next job
+    job.context["model"].Clear()
+    del job.context["model"].progressHandlerCallback
+    del job.context["model"]
+    del job.context
