@@ -1,17 +1,22 @@
 import concurrent.futures
+import threading
+import time
 from copy import deepcopy
 from dataclasses import asdict
 from os import PathLike
 from pathlib import Path
+from pprint import pformat
 from tempfile import TemporaryDirectory
 from typing import Union, Sequence, Mapping, List
 
 import pyqalx
+import numpy as np
 from OrcFxAPI import Model
 from pyqalx import QalxSession, Item, Group, Set
 
 from qalx_orcaflex import data_models
 from qalx_orcaflex.data_models import JobState
+from qalx_orcaflex.gui.batch.overview_main import run_gui, BatchOverview
 from qalx_orcaflex.helpers import (
     data_file_paths,
     required_results_from_yaml,
@@ -260,7 +265,7 @@ class OrcaFlexBatch:
         standard_meta = {
             "_class": "orcaflex.batch",
             "name": self.name,
-            "options": self.batch_options.__dict__,
+            "options": self.batch_options.to_valid_dict(),
             "state": "new",
         }
         if meta is None:
@@ -270,6 +275,13 @@ class OrcaFlexBatch:
             self.meta.update(meta)
         self.verbose = verbose
 
+    @classmethod
+    def get_and_check_progress(cls, batch_name, ofx_session=None):
+        """Given a name of a batch, run the GUI tool for batch progress"""
+        if ofx_session is None:
+            ofx_session = QalxOrcaFlex()
+        run_gui(ofx_session=ofx_session, batch_name=batch_name)
+
     def _add_case(self, case_n: int, job: Mapping) -> str:
         """adds a case to qalx
 
@@ -391,6 +403,14 @@ class OrcaFlexBatch:
         job_item_many = self.session.item.add_many(
             list(job_items.values())
         )  # add all the items
+        if job_item_many["errors"]:
+            # if there were errors here then we need to raise an exception
+            # otherwise the zipping back together will put things in the wrong place.
+            raise QalxOrcaFlexError(
+                f"There was an error adding one of the items."
+                f"More information can be found below:\n\n"
+                f"{pformat(job_item_many['errors'])}"
+            )
         job_items_zipped = dict(
             zip(job_items.keys(), job_item_many["items"])
         )  # re-pack
@@ -466,6 +486,73 @@ class OrcaFlexBatch:
             self._submit()
         self.temp_dir.__exit__(*args)
 
+    def check_progress(self):
+        """Run the check progress GUI in a separate thread"""
+        gui_thread = threading.Thread(target=run_gui, args=(self.session, self.name))
+        gui_thread.start()
+
+    def when_complete(self, interval=15, timeout=None, run_with_gui=True):
+        """
+        Returns a waiter for the batch that is instantiated with the same
+        arguments
+
+        :param interval:        The interval to use for the waiter
+        :param timeout:     The timeout of the waiter in sec
+        :param run_with_gui:    Flag to run or not the GUI for batch overview
+                                apart from the waiter
+        """
+        batches = BatchOverview.get_batches(self.session, self.name)
+        batch_overview = BatchOverview(self.session, self.name, batches)
+        return Waiter(self, batch_overview, interval, timeout, run_with_gui)
+
+
+class Waiter:
+    """
+    Implements "freezing" functionality that stops code execution until a batch
+    is complete
+
+    Instance attributes
+        batch:          The batch to wait on. The batch should have been added
+                        before the waiter is run
+        batch_overview: Instance of BatchOverview
+        interval:       The interval in seconds to use between checking for
+                        updates on the batch. Defaults to 15 seconds
+        timeout:        The timeout in seconds to use in order to override the
+                        waiting and just exit. This parameter is optional.
+        run_with_gui:   Boolean flag to run or not the batch overview GUI
+                        in addition. Defaults to True
+    """
+
+    def __init__(
+        self, batch, batch_overview, interval=15, timeout=None, run_with_gui=True
+    ):
+        self.batch = batch
+        self.batch_overview = batch_overview
+        self.index = len(self.batch_overview.batches) - 1
+        self.interval = interval
+        self.timeout = timeout
+        self.run_with_gui = run_with_gui
+        self.start_time = time.time()
+
+    def _check_timeout(self):
+        """Check if the timeout has been exceeded"""
+        if self.timeout is None:
+            return False
+        return (time.time() - self.start_time) > self.timeout
+
+    def __enter__(self):
+        if self.run_with_gui:
+            self.batch.check_progress()
+        all_complete = False
+        while not (all_complete or self._check_timeout()):
+            summary_df, _ = self.batch_overview.get_summary_df(self.index)
+            complete_array = summary_df.complete.to_numpy()
+            all_complete = np.all(complete_array)
+            time.sleep(self.interval)
+
+    def __exit__(self, exc_type, exc_val, exc_tb):
+        pass
+
 
 class QalxOrcaFlex(pyqalx.QalxSession):
     """we define a sublcass of QalxSession so there is no need to import from pyqalx
