@@ -16,13 +16,14 @@ processing of the simulation.
 
 """
 import ctypes as ct
-from dataclasses import dataclass, field, asdict
+from dataclasses import dataclass, field, asdict, fields
 from enum import Enum
 from typing import List, Mapping, Any, Tuple, Sequence
 
 import OrcFxAPI
 from pyqalx.core.entities import Item
 
+from .notifications import Notifications
 from ..video import DEFAULT_WIDTH, DEFAULT_HEIGHT
 
 OE_LINE_PT = {
@@ -40,6 +41,21 @@ OE_RADIAL = {
 }
 
 
+def rehydrate_dataclass(klass, d):
+    """
+    https://stackoverflow.com/a/54769644/11984516
+    Rehydrates a dictionary into the given dataclass.  Handles rehydrating nested dataclasses.
+    :param klass: The dataclasses class that will be rehydrated
+    :param d: The dict to use for the source data
+    """
+    try:
+        fieldtypes = {f.name: f.type for f in fields(klass)}
+        return klass(**{f: rehydrate_dataclass(fieldtypes[f], d[f]) for f in d})
+    except TypeError:
+        # TypeError: Not a dataclass field, probably an attribute on the class
+        return d
+
+
 @dataclass
 class OrcaFlexJobOptions:
     """Options for an OrcaFlex job
@@ -118,6 +134,9 @@ class BatchOptions:
     :param build_threads[int]: threads to use to build the batch(default=10)
     :param send_sim_to [list]: queues to send sims to when ``SimBot`` is finished
     :param send_batch_to [list]: queues to send batches to when ``BatchBot`` is finished
+    :param notifications [~data_models.notifications.Notifications]: An instance of Notifications which will detail
+    which notifications should be automatically sent out by the system.  (default=None)
+    :param timeout: How long should this batch be processed for before timing out in seconds
     """
 
     batch_queue: str
@@ -127,6 +146,13 @@ class BatchOptions:
     build_threads: int = 10
     send_sim_to: Sequence = field(default_factory=lambda: [])
     send_batch_to: Sequence = field(default_factory=lambda: [])
+    notifications: Notifications = None
+    timeout: int = None
+
+    def to_valid_dict(self):
+        if self.notifications:
+            self.notifications.validate()
+        return asdict(self)
 
 
 class ValidationError(Exception):
